#!/usr/bin/python
################################################################################
#	BdsDataClient1.py	BDS API example code for a BDS Data Client
#				T.Barnaby,	BEAM Ltd,	2012-12-19
################################################################################
#
# Example code to show getting data from the BDS system in a particular format.
#
import sys
import getopt
from bdslib import *

# Function to read some data
def bdsTest(bds):
	err		= BError();
	selection	= Selection();
	
	# Set up selection
	selection.startTime.setString("2008-01-01T00:00:00.000000");
	selection.endTime.setString("2008-01-01T00:01:00.000000");
	print "Selection: StartTime:", selection.startTime.getString(), "EndTime:", selection.endTime.getString();
	
	selection.channels.append(SelectionChannel("TT", "TSB01", "", ""));
#	bdsDumpSelection(selection);

	# Get list of all data available for the selection		
	(err, dataInfo) = bds.dataSearch(selection);
	if(err.getErrorNo()):
		return err.set(1, BString("Error: Searching for data: ") + err.getString());

	# We should now choose which set of data we would like from the list, here we just
	# choose all of the data.
	s = dataInfo.channels.size();
	if(s == 0):
		return err.set(1, "No data found");
	
#	bdsDumpDataInfo(dataInfo);
	
	# Open the data file for reading in IMS format
	(err, dataHandle) = bds.dataOpen(dataInfo, "r", "IMS", 0);
	if(err.getErrorNo()):
		return err;
	
	# Read the formatted data
	while(1):
#		print "Loop";
		(err, data) = bds.dataFormattedRead(dataHandle, 1024);
		if(err.getErrorNo()):
			return err;

		if(data.number() == 0):
			break;
		
#		print "Data0:", data[0];
#		fwrite(data.data(), 1, data.size(), stdout);

	return err;

def main():
	hostName = "localhost";
	bds = DataAccess();

	# Connect to the DataAccess service
	err = bds.connectService("//" + hostName + "/bdsDataAccess");
	if(err):
		print "Error: " + str(err) + "\n";
		return 1;

	# Connect to service
	err = bds.connect("test", "beam00");
	if(err):
		print "Error: " + str(err) + "\n";
		return 1;
	
	(err, version, name) = bds.getVersion();
	if(err):
		print "Error: " + str(err) + "\n";
		return 1;
	print "Version:" , version, "Name:", name;

	err = bdsTest(bds);
	if(err):
		print "Error:", err.getErrorNo(), err.getString();
		return 1;
		
	return 0;

if __name__ == "__main__":
	main();
