#!/usr/bin/python
################################################################################
#	BdsDataClient1e.cpp	BDS API example code for a Data Client
#				T.Barnaby,	BEAM Ltd,	2012-11-06
################################################################################
#
# Example code to show getting data from the BDS system in a particular format.
# This example uses exceptions.
#
import sys
import getopt
from bdslibe import *

# Function to read some data
def bdsTest(bds):
	err	= BError();
	selection = Selection();
	selectionChannel = SelectionChannel();
	dataInfo = DataInfo();
	dataHandle = DataHandle();
	
	# Set up selection
	selection.startTime.setString("2008-01-01T00:00:00.000000");
	selection.endTime.setString("2008-01-01T00:01:00.000000");
	print "Selection: StartTime:", selection.startTime.getString(), "EndTime:", selection.endTime.getString();
	
	selection.channels.append(SelectionChannel("TT", "TSB01", "", ""));
	bdsDumpSelection(selection);

	selectionChannel.network = "TT";
	for v in selection.channels:
		print v.network;

	# Get list of all data available for the selection		
	(err, dataInfo) = bds.dataSearch(selection);
	if(err.getErrorNo()):
		return err.set(1, BString("Error: Searching for data: ") + err.getString());

	# We should now choose which set of data we would like from the list, here we just
	# choose the first entry and get the data in appropriate format.
	s = dataInfo.channels.size();
	if(s == 0):
		return err.set(1, "No data found");
	
	bdsDumpDataInfo(dataInfo);
	
	(err, dataHandle) = bds.dataOpen(dataInfo, "r", "IMS", 0);
	if(err.getErrorNo()):
		return err;
	
	while(1):
		print "Loop";
		(err, data) = bds.dataFormattedRead(dataHandle, 1024);
		if(err.getErrorNo()):
			return err;

		print "DataSize:", data.number();
		if(data.number() == 0):
			break;
		
		print "Data0:", data[0];
#		fwrite(data.data(), 1, data.size(), stdout);

	return err;

# Function to read some data
def bdsTest1(bds):
	err	= BError();
	selection = Selection();
	dataInfo = DataInfo();
	
	# Set up selection
	selection.startTime.setString("2008-01-01T00:00:00.000000");
	selection.endTime.setString("2008-01-01T00:01:00.000000");
	selection.channels.append(SelectionChannel("TT", "TSB01", "", ""));
#	bdsDumpSelection(selection);

	# Get list of all data available for the selection		
	dataInfo = bds.dataSearch(selection);

	# We should now choose which set of data we would like from the list, here we just
	# choose the first entry and get the data in appropriate format.
	s = dataInfo.channels.size();
	if(s == 0):
		return err.set(1, "No data found");
	
#	bdsDumpDataInfo(dataInfo);
	
	dataHandle = bds.dataOpen(dataInfo, "r", "API", 0);
	
	blockNumber = 0;
	while(1):
		print "Loop";
		try:
			data = bds.dataGetBlock(dataHandle, 0, 1, blockNumber);
		except ExceptionBError as e:
			if(e.number == ErrorEndOfData):
				return err;
			else:
				raise e;

		print "DataChannels:", data.channelData.size();
		print "Data0:", data.channelData[0][0];
		blockNumber += 1;

	return err;

def main():
	hostName = "localhost";
	bds = DataAccess();

	try:
		# Connect to the DataAccess service
		err = bds.connectService("//" + hostName + "/bdsDataAccess");

		# Connect to service
		bds.connect("test", "beam00");

		(version, name) = bds.getVersion();
		print "Version:" , version, "Name:", name;

		bdsTest1(bds);
		
	except ExceptionBError as e:
		print "Execption", type(e);
		print "Execption", e.args;
		print "Execption", e.number, e.string;
		sys.exit(1);

	return 0;

if __name__ == "__main__":
	main();
