#!/usr/bin/python
################################################################################
#	BdsDataClient2.py	BDS API example code for a BDS Data Client
#				T.Barnaby,	BEAM Ltd,	2012-12-19
################################################################################
#
# Example code to show getting data from the BDS system in raw data format.
#
import sys
import getopt
from bdslib import *

# Function to read some data
def bdsTest(bds):
	err	= BError();
	selection = Selection();
	dataInfo = DataInfo();
	
	# Set up selection
	selection.startTime.setString("2008-01-01T00:00:00.000000");
	selection.endTime.setString("2008-01-01T00:01:00.000000");
	selection.channels.append(SelectionChannel("TT", "TSB01", "", ""));
#	bdsDumpSelection(selection);

	# Get list of all data available for the selection		
	(err, dataInfo) = bds.dataSearch(selection);
	if(err.getErrorNo()):
		return err.set(1, BString("Error: Searching for data: ") + err.getString());

	# We should now choose which set of data we would like from the list, here we just
	# choose the first entry and get the data in appropriate format.
	s = dataInfo.channels.size();
	if(s == 0):
		return err.set(1, "No data found");
	
#	bdsDumpDataInfo(dataInfo);
	
	(err, dataHandle) = bds.dataOpen(dataInfo, "r", "API", 0);
	if(err.getErrorNo()):
		return err;
	
	blockNumber = 0;
	while(1):
#		print "Loop";
		(err, data) = bds.dataGetBlock(dataHandle, 0, 1, blockNumber);
		if(err.getErrorNo()):
			return err;

#		print "DataChannels:", data.channelData.size();
		
		print "Data0:", data.channelData[0][0];
		blockNumber += 1;

	return err;

def main():
	hostName = "localhost";
	bds = DataAccess();

	# Connect to the DataAccess service
	err = bds.connectService("//" + hostName + "/bdsDataAccess");
	if(err):
		print "Error: " + str(err) + "\n";
		return 1;

	# Connect to service
	err = bds.connect("test", "beam00");
	if(err):
		print "Error: " + str(err) + "\n";
		return 1;
	
	(err, version, name) = bds.getVersion();
	if(err):
		print "Error: " + str(err) + "\n";
		return 1;
	print "Version:" , version, "Name:", name;

	err = bdsTest(bds);
	if(err):
		print "Error:", err.getErrorNo(), err.getString();
		return 1;
		
	return 0;

if __name__ == "__main__":
	main();
