/*******************************************************************************
 *	BdsDataClient3.cpp	BDS API example code for a Data Client
 *				T.Barnaby,	BEAM Ltd,	2008-09-02
 *******************************************************************************
 *
 *	This is a very basic example of using the BdsApi from a data access
 *	perspective. It is designed to give an overview of using the API.
 *	This program gets information on the data channels.
 */
#include <iostream>
#include <iomanip>
#include <stdio.h>
#include <BdsD.h>
#include <BdsC.h>

using namespace Bds;
using namespace std;


// Function to read some data
BError bdsTest(DataAccess& bds){
	BError			err;
	Selection		selection;
	DataInfo		dataInfo;
	ChannelInfos		channelInfos;
	BIter			i;
	BUInt			c;
	
	// Set up selection
	selection.startTime.setString("2008-01-03T00:00:00.000000");
	selection.endTime.setString("2008-01-03T00:01:00.000000");
	selection.channels.append(SelectionChannel("BN", "EKA", "", ""));

	// Get list of all data available for the selection		
	if(err = bds.dataSearch(selection, dataInfo)){
		return err.set(1, BString("Error: Searching for data: ") + err.getString());
	}

	// We should now choose which set of data we would like from the list, here we just
	// choose the first entry and get the data in appropriate format.
	if(!dataInfo.channels.size())
		return err.set(1, "No data found");
	
	if(err = bds.dataGetChannelInfo(dataInfo, channelInfos)){
		return err;
	}

	// This displays some of the information available	
	for(c = 0; c < channelInfos.channels.size(); c++){
		ChannelInfo&	ci = channelInfos.channels[c][0];
		
		cout << (c + 1) << ": Station: " << ci.station.name << " Channel: " << ci.channel.channel << "\n";
		cout << "    " << "Calibration: " << ci.calibration.calibrationFactor << " Frequency: " << ci.calibration.calibrationFrequency <<
			" Units: " << ci.calibration.calibrationUnits << "\n";
		cout << "    " << "Instrument: " << ci.digitiser.name << "\n";
	}
	
	
	return err;
}

int main(int argc, char** argv){
	BError			err;
	BString			hostName;
	DataAccess		bds;

	hostName = getenv("BDS_HOST");
	if(hostName == "")
		hostName = "localhost";
	
	if(argc == 2)
		hostName = argv[1];

	// Connect to the DataAccess service
	if(err = bds.connectService(BString("//") + hostName + "/bdsDataAccess")){
		cerr << "Error: " << err.getString() << "\n";
		return 1;
	}
	
	// Connect to service
	if(!err)
		err = bds.connect("test", "beam00");
	
	// Run a normal data gathering as a normal data access client would.
	if(!err)
		err = bdsTest(bds);
	
	if(err){
		cerr << "Error: " << err.getString() << "\n";
		return 1;
	}
		
	return 0;
}
