#!/usr/bin/python
################################################################################
#	BdsDataClient2.py	BDS API example code for a BDS Data Client
#				T.Barnaby,	BEAM Ltd,	2012-12-19
################################################################################
#
# This example shows how to obtain some MetaData from the BDS system for an actual
# set of seismic data..
# The example assumes the BDS test data is installed.
#
import sys
import getopt
from bdslib import *

# Function to read some data
def bdsTest(bds):
	err	= BError();
	selection = Selection();
	dataInfo = DataInfo();
	
	# Set up selection
	selection.startTime.setString("2008-01-01T00:00:00.000000");
	selection.endTime.setString("2008-01-01T00:01:00.000000");
	selection.channels.append(SelectionChannel("TT", "TSB01", "", ""));
#	bdsDumpSelection(selection);

	# Get list of all data available for the selection		
	(err, dataInfo) = bds.dataSearch(selection);
	if(err.getErrorNo()):
		return err.set(1, BString("Error: Searching for data: ") + err.getString());

	# We should now choose which set of data we would like from the list, here we just
	# choose the first entry and get the data in appropriate format.
	s = dataInfo.channels.size();
	if(s == 0):
		return err.set(1, "No data found");
	
#	bdsDumpDataInfo(dataInfo);

	(err, channelInfos) = bds.dataGetChannelInfo(dataInfo);
	if(err.getErrorNo()):
		return err;

	# This displays some of the information available	
	for c in range(0, channelInfos.channels.size()):
		ci = channelInfos.channels[c][0];
		
		print str(c + 1) + ": Station: " + ci.station.name + " Channel: " + str(ci.channel.channel);
		print "    " + "Calibration: " + str(ci.calibration.calibrationFactor) + " Frequency: " + str(ci.calibration.calibrationFrequency) + " Units: " + ci.calibration.calibrationUnits;
		print "    " + "Instrument: " + ci.digitiser.name;

def main():
	hostName = "localhost";
	bds = DataAccess();

	# Connect to the DataAccess service
	err = bds.connectService("//" + hostName + "/bdsDataAccess");
	if(err):
		print "Error: " + str(err) + "\n";
		return 1;

	# Connect to service
	err = bds.connect("test", "beam00");
	if(err):
		print "Error: " + str(err) + "\n";
		return 1;
	
	(err, version, name) = bds.getVersion();
	if(err):
		print "Error: " + str(err) + "\n";
		return 1;
	print "Version:" , version, "Name:", name;

	err = bdsTest(bds);
	if(err):
		print "Error:", err.getErrorNo(), err.getString();
		return 1;
		
	return 0;

if __name__ == "__main__":
	main();
