/*******************************************************************************
 *	BdsDataFile2.cpp	BDS API example code for a Data File Access
 *				T.Barnaby,	BEAM Ltd,	2014-11-28
 *******************************************************************************
 *
 *	This Example code opens a Response file, gets information from the file.
 */
#include <stdio.h>
#include <BdsDataFileResponse.h>

using namespace Bds;
using namespace std;

int main(int argc, char** argv){
	BError			err;
	DataFileResponse	f;
	BString			filename = "response.resp";
	ChannelInfos		ci;
	BUInt			c;
	
	// Open the data file
	f.open(filename, "r");
	f.setFormat("RESPONSE");
	
	err = f.getMetaData(ci);
	if(err){
		printf("Failed to extract Ps and Zs from file %s: %s\n", filename.retStr(), err.getString().retStr()); 
		return 1;
	}

	printf("Number of channels of info: %d\n", ci.channels.number());
	if(ci.channels.number()){
		printf("Number of segments of info in channel 0: %d\n", ci.channels[0].number());
		if(ci.channels[0].number()){
			printf("Number of Responses: %d\n", ci.channels[0][0].responses.number());
			BListLoop(ci.channels[0][0].responses, i){
				Response&	r = ci.channels[0][0].responses[i];
				
				printf("Response Type: %s\n", r.type.retStr());
				if(r.type == "PoleZero"){
					for(c = 0; c < r.poleZeros.poles.number(); c++){
						printf("Pole: %f %f\n", r.poleZeros.poles[c].real(), r.poleZeros.poles[c].imag());
					}
					for(c = 0; c < r.poleZeros.zeros.number(); c++){
						printf("Zero: %f %f\n", r.poleZeros.zeros[c].real(), r.poleZeros.zeros[c].imag());
					}
				}
			}
		}
	}

	if(err){
		cerr << "Error: " << err.getString() << "\n";
		return 1;
	}
		
	return 0;
}
