#!/usr/bin/python
################################################################################
#	BdsMetaData1.py		BDS API example code for a BDS MetaData Client
#				T.Barnaby,	BEAM Ltd,	2012-12-19
################################################################################
#
# This example shows how to obtain some MetaData from the BDS system.
# In this example it gets some information on a Station.
# The example assumes the BDS test data is installed.
#
import sys
import getopt
from bdslib import *

# Function to read display info on Station
def bdsTest(bds):
	err	= BError();
	selection = Selection();
	
	# Set up selection
	selection.startTime.setString("2008-01-01T00:00:00.000000");
	selection.endTime.setString("2008-01-01T00:01:00.000000");
	selection.channels.append(SelectionChannel("TT", "TSA", "", ""));
#	bdsDumpSelection(selection);

	# Get list of stations available
	(err, stations) = bds.stationGetList(selection);
	if(err):
		return err.set(1, "Error: Getting stations: " + err.getString());

	# This displays some of the information available
	for s in stations:
		print "Station: " + s.name + " Type: " + s.type;
		print "    " + "Description: " + s.description + " Number of station/channels " + str(s.channels.number());

	return err;

def main():
	hostName = "localhost";
	
	# Create DataAccess object to connect to BDS Server
	bds = DataAccess();

	# Connect to the DataAccess service
	err = bds.connectService("//" + hostName + "/bdsDataAccess");
	if(err):
		print "Error: " + str(err) + "\n";
		return 1;

	# Connect to service
	err = bds.connect("test", "beam00");
	if(err):
		print "Error: " + str(err) + "\n";
		return 1;
	
	(err, version, name) = bds.getVersion();
	if(err):
		print "Error: " + str(err) + "\n";
		return 1;
	print "Version:" , version, "Name:", name;

	err = bdsTest(bds);
	if(err):
		print "Error:", err.getErrorNo(), err.getString();
		return 1;
		
	return 0;

if __name__ == "__main__":
	main();
