#!/usr/bin/python
################################################################################
#	BdsMetaData2.py		BDS API example code for a BDS MetaData Client
#				T.Barnaby,	BEAM Ltd,	2012-12-19
################################################################################
#
# This example shows how to obtain some MetaData from the BDS system.
# In this example it gets some channel response information.
# The example assumes the BDS test data is installed.
#
import sys
import getopt
from bdslib import *

# Function to get Response for a channel
def bdsGetResponse(bds):
	err	= BError();
	selection = Selection();
	
	# Set up selection
	selection.startTime.setString("2008-01-01T00:00:00.000000");
	selection.endTime.setString("2008-01-01T00:01:00.000000");
	selection.channels.append(SelectionChannel("TT", "TSB01", "", ""));
#	bdsDumpSelection(selection);

	# Get list of channels that match
	(err, responses) = bds.responseGetList(selection);
	if(err):
		return err.set(1, "Error: Getting channels: " + err.getString());

	# This displays some of the information available
	for r in responses:
		print "Response: " + r.network + ":" + r.station + ":" + r.channel + " Name: " + r.name + " Type: " + r.type;

		print "\tPoles:";
		for i in r.poleZeros.poles:
			print "\t", i.real, i.imag;

		print "\tZeros:";
		for i in r.poleZeros.zeros:
			print "\t", i.real, i.imag;
	
	

	return err;

def bdsGetCalibration(bds):
	err	= BError();
	selection = Selection();
	
	# Set up selection
	selection.startTime.setString("2008-01-01T00:00:00.000000");
	selection.endTime.setString("2008-01-01T00:01:00.000000");
	selection.channels.append(SelectionChannel("TT", "TSB01", "", ""));
#	bdsDumpSelection(selection);

	# Get list of channels that match
	(err, calibrations) = bds.calibrationGetList(selection);
	if(err):
		return err.set(1, "Error: Getting channels: " + err.getString());

	# This displays some of the information available
	for c in calibrations:
		print "Calibration: " + c.network + ":" + c.station + ":" + c.channel + " Name: " + c.name + "CalibrationFactor:", c.calibrationFactor;
	

	return err;
	
def bdsGetInstruments(bds):
	err	= BError();
	selection = Selection();
	
	# Set up selection
	selection.startTime.setString("2008-01-01T00:00:00.000000");
	selection.endTime.setString("2008-01-01T00:01:00.000000");
	selection.channels.append(SelectionChannel("TT", "TSB01", "", ""));
#	bdsDumpSelection(selection);

	# Get list of channels that match
	(err, channelInstruments) = bds.channelInstrumentGetList(selection);
	if(err):
		return err.set(1, "Error: Getting channelInstruments: " + err.getString());

	# This displays some of the information available
	for c in channelInstruments:
		print "ChannelInstruments: " + "ChannelId::" + str(c.channelId) + " SensorId: " + str(c.sensorId) + " DigitiserId: " + str(c.digitiserId);
		(err, s) = bds.sensorGet(c.sensorId);
		if(err):
			return err.set(1, "Error: Getting sensor: " + err.getString());
		print "\tSensor: ", s.name, " Type: ", s.type;

		(err, d) = bds.digitiserGet(c.digitiserId);
		if(err):
			return err.set(1, "Error: Getting digitiserId: " + err.getString());		
		print ("\tDigitiser: %s Type: %s") % (d.name, d.type);
		
	return err;

def main():
	hostName = "localhost";
	
	# Create DataAccess object to connect to BDS Server
	bds = DataAccess();

	# Connect to the DataAccess service
	err = bds.connectService("//" + hostName + "/bdsDataAccess");
	if(err):
		print "Error: " + str(err) + "\n";
		return 1;

	# Connect to service
	err = bds.connect("test", "beam00");
	if(err):
		print "Error: " + str(err) + "\n";
		return 1;

	# Run the tests
	err = bdsGetResponse(bds);
	if(err):
		print "Error:", err.getErrorNo(), err.getString();
		return 1;
		
	err = bdsGetCalibration(bds);
	if(err):
		print "Error:", err.getErrorNo(), err.getString();
		return 1;

	err = bdsGetInstruments(bds);
	if(err):
		print "Error:", err.getErrorNo(), err.getString();
		return 1;


	return 0;

if __name__ == "__main__":
	main();
