#!/usr/bin/python
################################################################################
#	BdsMetaData2e.py		BDS API example code for a BDS MetaData Client
#				T.Barnaby,	BEAM Ltd,	2012-12-19
################################################################################
#
# This example shows how to obtain some MetaData from the BDS system.
# In this example it gets some channel response information.
# The example assumes the BDS test data is installed.
# The example uses exceptions.
#
import sys
import getopt
from bdslibe import *

# Function to get Response for a channel
def bdsGetResponse(bds):
	selection = Selection();
	
	# Set up selection
	selection.startTime.setString("2008-01-01T00:00:00.000000");
	selection.endTime.setString("2008-01-01T00:01:00.000000");
	selection.channels.append(SelectionChannel("TT", "TSB01", "", ""));
#	bdsDumpSelection(selection);

	# Get list of channels that match
	responses = bds.responseGetList(selection);

	# This displays some of the information available
	for r in responses:
		print "Response: " + r.network + ":" + r.station + ":" + r.channel + " Name: " + r.name + " Type: " + r.type;

		print "\tPoles:";
		for i in r.poleZeros.poles:
			print "\t", i.real, i.imag;

		print "\tZeros:";
		for i in r.poleZeros.zeros:
			print "\t", i.real, i.imag;
	

def bdsGetCalibration(bds):
	selection = Selection();
	
	# Set up selection
	selection.startTime.setString("2008-01-01T00:00:00.000000");
	selection.endTime.setString("2008-01-01T00:01:00.000000");
	selection.channels.append(SelectionChannel("TT", "TSB01", "", ""));
#	bdsDumpSelection(selection);

	# Get list of channels that match
	calibrations = bds.calibrationGetList(selection);

	# This displays some of the information available
	for c in calibrations:
		print "Calibration: " + c.network + ":" + c.station + ":" + c.channel + " Name: " + c.name + "CalibrationFactor:", c.calibrationFactor;
	
	
def bdsGetInstruments(bds):
	selection = Selection();
	
	# Set up selection
	selection.startTime.setString("2008-01-01T00:00:00.000000");
	selection.endTime.setString("2008-01-01T00:01:00.000000");
	selection.channels.append(SelectionChannel("TT", "TSB01", "", ""));
#	bdsDumpSelection(selection);

	# Get list of channels that match
	channelInstruments = bds.channelInstrumentGetList(selection);

	# This displays some of the information available
	for c in channelInstruments:
		print "ChannelInstruments: " + "ChannelId::" + str(c.channelId) + " SensorId: " + str(c.sensorId) + " DigitiserId: " + str(c.digitiserId);
		s = bds.sensorGet(c.sensorId);
		print "\tSensor: ", s.name, " Type: ", s.type;

		d = bds.digitiserGet(c.digitiserId);
		print ("\tDigitiser: %s Type: %s") % (d.name, d.type);

def main():
	hostName = "localhost";
	
	# Create DataAccess object to connect to BDS Server
	bds = DataAccess();

	try:
		# Connect to the DataAccess service
		bds.connectService("//" + hostName + "/bdsDataAccess");

		# Connect to service
		bds.connect("test", "beam00");

		# Run the tests
		bdsGetResponse(bds);
		bdsGetCalibration(bds);
		bdsGetInstruments(bds);

	except ExceptionBError as e:
		print "Execption", e.number, e.string;
		sys.exit(1);

	return 0;

if __name__ == "__main__":
	main();
