#!/usr/bin/python
################################################################################
#	BdsMetaData3.py		BDS API example code for a BDS MetaData Client
#				T.Barnaby,	BEAM Ltd,	2012-12-19
################################################################################
#
# This example shows how to modify some MetaData information.
# The example assumes the BDS test data is installed.
#
#
import sys
import getopt
from bdslib import *

# Function to get Response for a channel
def bdsModResponse(bds):
	err	= BError();
	selection = Selection();
	
	# Set up selection
	selection.startTime.setString("2008-01-01T00:00:00.000000");
	selection.endTime.setString("2008-01-01T00:01:00.000000");
	selection.channels.append(SelectionChannel("TT", "TSB01", "", ""));
#	bdsDumpSelection(selection);

	# Get list of channels that match
	(err, responses) = bds.responseGetList(selection);
	if(err):
		return err.set(1, "Error: Getting channels: " + err.getString());

	# Prints and mods this response
	if(len(responses)):
		r = responses[0];
		print "Response: " + r.network + ":" + r.station + ":" + r.channel + " Name: " + r.name + " Type: " + r.type;

		print "\tPoles:";
		for i in r.poleZeros.poles:
			print "\t", i.real, i.imag;

		print "\tZeros:";
		for i in r.poleZeros.zeros:
			print "\t", i.real, i.imag;
		
		r.poleZeros.poles[0] += 10;
		(err, id) = bds.responseUpdate(0, r);
		if(err):
			return err.set(1, "Error: Setting Response: " + err.getString());

	return err;

def bdsModCalibration(bds):
	err	= BError();
	selection = Selection();
	
	# Set up selection
	selection.startTime.setString("2008-01-01T00:00:00.000000");
	selection.endTime.setString("2008-01-01T00:01:00.000000");
	selection.channels.append(SelectionChannel("TT", "TSB01", "", ""));
#	bdsDumpSelection(selection);

	# Get list of channels that match
	(err, calibrations) = bds.calibrationGetList(selection);
	if(err):
		return err.set(1, "Error: Getting channels: " + err.getString());

	# This changes the Calibration factor
	if(len(calibrations)):
		c = calibrations[0];
		print "Calibration: " + c.network + ":" + c.station + ":" + c.channel + " Name: " + c.name + "CalibrationFactor:", c.calibrationFactor;
	
		c.calibrationFactor += 10e-9;
		(err, id) = bds.calibrationUpdate(0, c);
		if(err):
			return err.set(1, "Error: Setting Calibration: " + err.getString());
	return err;
	
def bdsModInstruments(bds):
	err	= BError();
	selection = Selection();
	
	# Set up selection
	selection.startTime.setString("2008-01-01T00:00:00.000000");
	selection.endTime.setString("2008-01-01T00:01:00.000000");
	selection.channels.append(SelectionChannel("TT", "TSB01", "", ""));
#	bdsDumpSelection(selection);

	# Get list of channels that match
	(err, channelInstruments) = bds.channelInstrumentGetList(selection);
	if(err):
		return err.set(1, "Error: Getting channelInstruments: " + err.getString());

	# This displays some of the information available
	if(len(channelInstruments)):
		c = channelInstruments[0];

		print "ChannelInstruments: " + "ChannelId::" + str(c.channelId) + " SensorId: " + str(c.sensorId) + " DigitiserId: " + str(c.digitiserId);
		(err, s) = bds.sensorGet(c.sensorId);
		if(err):
			return err.set(1, "Error: Getting sensor: " + err.getString());
		print "\tSensor: ", s.name, " Type: ", s.type;

		(err, d) = bds.digitiserGet(c.digitiserId);
		if(err):
			return err.set(1, "Error: Getting digitiserId: " + err.getString());		
		print ("\tDigitiser: %s Type: %s") % (d.name, d.type);
		
		# Create a new Sensor cloning for existing
		snew = s;
		(err, id) = bds.sensorUpdate(1, snew);
		if(err):
			return err.set(1, "Error: Setting Sensor: " + err.getString());

		print "NewSensorId:", id;
		c.sensorId = id;
		(err, id) = bds.channelInstrumentUpdate(0, c);
		if(err):
			return err.set(1, "Error: Setting Channel Instrument: " + err.getString());
		
	return err;

def bdsModCalibrationTransaction(bds):
	err	= BError();
	selection = Selection();
	
	# Set up selection
	selection.startTime.setString("2008-01-01T00:00:00.000000");
	selection.endTime.setString("2008-01-01T00:01:00.000000");
	selection.channels.append(SelectionChannel("TT", "TSA", "", ""));
#	bdsDumpSelection(selection);

	# Get list of channels that match
	(err, calibrations) = bds.calibrationGetList(selection);
	if(err):
		return err.set(1, "Error: Getting channels: " + err.getString());

	print "Num Calibtraions:", len(calibrations);

	# This changes the Calibration factors for all channels
	
	# Start a new Transaction
	bds.transactionStart();

	for c in calibrations:
		print "Calibration: " + c.network + ":" + c.station + ":" + c.channel + " Name: " + c.name + "CalibrationFactor:", c.calibrationFactor;
	
		c.calibrationFactor += 10e-9;
		(err, id) = bds.calibrationUpdate(0, c);
		if(err):
			return err.set(1, "Error: Setting Calibration: " + err.getString());

	# Abort transaction with and error
	bds.transactionEnd(1);

	return err;

def main():
	hostName = "localhost";
	
	# Create DataAccess object to connect to BDS Server
	bds = AdminAccess();

	# Connect to the DataAccess service
	err = bds.connectService("//" + hostName + "/bdsAdminAccess");
	if(err):
		print "Error: " + str(err);
		return 1;

	# Connect to service
	err = bds.connect("testAdmin", "beam00");
	if(err):
		print "Error: " + str(err);
		return 1;

	# Run the tests
	err = bdsModResponse(bds);
	if(err):
		print "Error:", err.getErrorNo(), err.getString();
		return 1;

	err = bdsModCalibration(bds);
	if(err):
		print "Error:", err.getErrorNo(), err.getString();
		return 1;

	err = bdsModInstruments(bds);
	if(err):
		print "Error:", err.getErrorNo(), err.getString();
		return 1;

	err = bdsModCalibrationTransaction(bds);
	if(err):
		print "Error:", err.getErrorNo(), err.getString();
		return 1;

	return 0;

if __name__ == "__main__":
	main();
