/*******************************************************************************
 *	BdsDataFileCd.h	BDRS Data File Access
 *			T.Barnaby,	BEAM Ltd,	2011-06-01
 *******************************************************************************
 */
#ifndef BdsDataFileCd_H
#define	BdsDataFileCd_H

#include <BdsDataFile.h>

namespace Bds {

struct CdChannel_1v0 {
	BUInt8		auth;
	BUInt8		compress;
	BUInt8		spare0;
	BUInt8		spare1;
	BFloat32	calibrationFactor;
	BFloat32	calibrationPeriod;
	char		name[16];
	BUInt32		channel;
};

struct CdDataFormatFrame_1v0 {
	BUInt32		frameType;
	BUInt32		frameLength;
	BUInt32		maxFrameLength;
	BUInt32		numChannels;
	BUInt32		period;
	CdChannel_1v0	channels[100];
};

class CdDataChannel {
public:
	BString		station;
	BString		channel;
	char		mode[24];
	char		status[32];
	BTimeStamp	startTime;
	BUInt32		period;
	BUInt32		numSamples;
	BUInt32		dataSize;
	BUInt8*		data;
};

class CdPacketData {
public:
	BUInt32			frameType;
	BUInt32			trailerOffset;
	char			creator[8];
	char			destination[8];
	BUInt64			sequenceNum;
	BUInt32			series;

	BUInt32			numChannels;
	BUInt32			period;
	BTimeStamp		startTime;
	BArray<CdDataChannel>	channels;
	
	BUInt32			authKey;
	BUInt32			authSize;
	char*			auth;
	BUInt64			crc;
};

class CdFlag {
public:
			CdFlag(){ dead = zeroed = 0; };
	int		dead;
	int		zeroed;
};

class DataFileCd : public DataFile{
public:
				DataFileCd();

	int			getFeatures();						///< Get bitmask of supported features
	DataOrder		getDataOrder();

	BError			getInfo(DataInfo& dataInfo, DataFileOptions options, BList<DataError>& errors);
	BError			readData(BUInt32 channel, BUInt segment, BUInt32 blockNumber, DataBlock& data);

	static DataFormat	getFormats();
private:
	BError			readPacket_1v0(int validateNext);
	BError			readPacket_1v0_Scan(BUInt64& pos);
	BError			readBlock_1v0(BUInt32 channel, BUInt64 pos, DataBlock& data, int addInfo, int validateNext);
	BError			readPacket_1v1(int validateNext);
	BError			readPacket_1v1_Scan(BUInt64& pos);
	BError			readBlock_1v1(BUInt32 channel, BUInt64 pos, DataBlock& data, int addInfo, int validateNext);
	DataError		getBlockReorderInfo();			///< Returns information on any block re-ordering performed

	DataFileOptions			ooptions;			///< The current read options
	bool				ohasYear2000Blocks;		///< Flag to indicate year 2000 blocks
	DataInfo			odataInfo;			///< The DataInfo
	BBufferStore			opacket;			///< Current data packet
	BBufferStore			opacketNext;			///< Next data packet
	BArray<BArray<DataBlockPos> >	oblockPositions;		///< Block positions for each channel
	
	int				odataFormat;			///< CD1.0 current DataFormat in use
	BArray<CdDataFormatFrame_1v0>	odataFormats;			///< CD1.0 data formats
	BArray<CdFlag>			ochannelFlags;			///< Flags per channel;
};

}
#endif
