/*******************************************************************************
 *	BdsDataFileSeed.h	BDRS Data File Access
 *			T.Barnaby,	BEAM Ltd,	2009-12-18
 *******************************************************************************
 */
#ifndef BdsDataFileSeed_H
#define	BdsDataFileSeed_H

#include <BdsDataFile.h>
#include <BdsSeedTypes.h>
#include <BMutex.h>
#include <mseed/libmseed.h>
#include <mseed/unpackdata.h>

namespace Bds {

class DataFileSeed : public DataFile{
public:
				DataFileSeed();
				~DataFileSeed();
	
	BError			close();

	DataOrder		getDataOrder();
	int			getFeatures();						///< Get bitmask of supported features

	BError			getInfo(DataInfo& dataInfo, DataFileOptions options, BList<DataError>& errors);
	BError			readData(BUInt32 channel, BUInt segment, BUInt32 blockNumber, DataBlock& data);

	BError			setInfo(DataInfo& dataInfo, ChannelInfos& channelInfos, WriteOptionsList options = WriteOptionSensorData);
	BError			start(BUInt channel, BUInt segment);
	BError			writeData(DataBlock& data);
	BError			end();	

	void			msrFileWrite(void* data, int len);

	static DataFormat	getFormats();
	static BError		omsrErr;			///< MSR processing error
	static int		onoLock;			///< Disable libmseed locking

private:
	BError			processControl();
	BError			processBlockette();
	BError			processData(DataBlock& data, int addInfo);
	BError			setBlockSize(BUInt blockSize);
	BError			readBlockData(char* buf, BUInt numBytes);
	BError			readBlock(BUInt blockNumber, char& type, char& cont, BUInt& seq);
	BError			writeVolumeHeader(int rl);
	BError			writeTimeSpans();
	BError			writeBlockData(char type, const char* buf, BUInt numBytes);
	BError			writeFlush();
	BString			unitsConvert(BString units);
	int			getChannel(BString name);
	void			sortChannels();			///< Sort channels into station order
	
	int			addCode(BString str);
	int			addUnitCode(BString str);

	DataFileOptions		oreadOptions;
	DataInfo		odataInfo;
	ChannelInfos		ochannelInfos;
	double			osampleRate;
	BArray<BArray<DataBlockPos> >	oblockPositions;

	BUInt32			oblockSize;			///< The block size in use
	char*			oblock;				///< Current block data
	BUInt			oblockNumber;			///< Current block number
	BUInt			opos;				///< Current data position within block
	BUInt			oblocketteNumber;		///< Current blockette number
	BUInt			ologNumber;

	// SEED information
	BArray<BString>		ocodes;				///< List of coded text strings
	BArray<BString>		ounitCodes;			///< List of coded units strings

	// SEED import information
	BdsSeedType10		otype10;			///< Seed MetaData
	BdsSeedType11		otype11;
	BdsSeedType30		otype30;
	BdsSeedType52		otype52;
	BArray<BdsSeedType30>	oseedDataFormats;		///< Seed data formats
	BArray<BdsSeedStation>	oseedStations;			///< Seed stations from MetaData
	BArray<BdsSeedType52>	oseedChannels;			///< Seed channels from MetaData
	
	BArray<BString>		odataChannels;			///< The Seed data channels from data blocks
	BArray<int>		odataChannelFormats;		///< The data formats of each channel in libmseed form

	// SEED export information
	BArray<BInt32>		odataInt32;			///< Write data buffer
	BArray<BFloat32>	odataFloat32;			///< Write data buffer
	
	BUInt			odataChannel;			///< The current channel being written
	BUInt			odataSegment;			///< The current channel being written
	BUInt			odataTimeSpanBlock;		///< The block the time span list is in
	BDict<int>		ostationBlockNumbers;		///< The station block numbers
	BArray<BArray<BUInt32> >	odataStartBlocks;	///< Per data channel/segment start blocks
	BArray<BArray<BUInt32> >	odataEndBlocks;		///< Per data channel/segment end blocks

	static BMutex		olock;				///< Lock the libmseed library
	MSRecord*		omsr;				///< Libmseed data block processing
};

}
#endif
