/*******************************************************************************
 *	BdsDataFileTapeDigitiser.h	DataFileTapeDigitiser
 *			T.Barnaby,	BEAM Ltd,	2006-03-17
 *******************************************************************************
 */
#ifndef BdsDataFileTapeDigitiser_H
#define BdsDataFileTapeDigitiser_H	1

#include <BTypes.h>
#include <BError.h>
#include <BFile.h>
#include <BEntry.h>
#include <BBuffer.h>
#include <BDict.h>
#include <BdsDataFile.h>

namespace Bds {

/// This class implements the TapeDigitiser's file output conversion and storeing system
class DataFileTapeDigitiser : public DataFile {
public:
				DataFileTapeDigitiser();
	
	BError			open(BString fileName, BString mode);		///< Open the file for reading or writing

				// Read routines
	BError			getInfo(DataInfo& dataInfo, DataFileOptions options, BList<DataError>& errors);
	BError			readData(BUInt32 channel, BUInt segment, BUInt32 blockNumber, DataBlock& data);
	
	static DataFormat	getFormats();
private:
	uint32_t		blockSize();					///< Returns the data block size in bytes
	uint32_t		blockNumSamples();				///< Returns the number of samples of data in a block
	BError			readHeader(BDictString& header);			///< Read the file header info
	uint32_t		computeChecksum(void* data, int nBytes);

	BDictString		oheader;
	off64_t			oheaderSize;
	uint32_t		oblockSize;
	uint32_t		oblockNumSamples;
	double			osampleRate;
	BBuffer			obuffer;					///< Data block buffer
	BString			oversion;
	DataInfo		odataInfo;
};

}
#endif
