/*******************************************************************************
 *	BdsLib.h	Data File Access
 *			T.Barnaby,	BEAM Ltd,	2008-12-03
 *******************************************************************************
 */
#ifndef BdsLib_H
#define	BdsLib_H

#include <BdsD.h>
#include <BdsC.h>
#include <BString.h>

namespace Bds {

const int	NetworkNameLen = 3;		// Maximum Network name length
const int	StationNameLen = 5;		// Maximum Station name length
const int	ChannelTypeLen = 3;		// Maximum Channel type name length
const int	ChannelAuxLen = 2;		// Maximum Channel Aux length
const int	SourceLen = 16;			// Maximum Source length

/// Response Editor object
class ResponseEdit : public Response {
public:
			ResponseEdit(const Response& response);
			~ResponseEdit();
	
	BString		getString();
	void		setString(BString str);
};

class DataError {
public:
			DataError();
			DataError(int errorNumber, BString title, BString filename, BTimeStamp startTime, BTimeStamp endTime, DataInfo& dataInfo, BUInt channel, BString description, BString user = "");
	DataError&	set(int errorNumber, BString title, BString importFilename, BTimeStamp startTime, BTimeStamp endTime, DataInfo& dataInfo, BUInt channel, BString description, BString user = "");
	void		mergeDataInfo(DataInfo& dataInfo, BUInt channel);

	int		getErrorNumber() const;				///< Get The error number
	BString		getTitle() const;				///< Get the title

	BError		setString(BString str);				///< Set from string
	BError		setStringUser(BString str, BString user);	///< Set from string given by user on command line
	BString		getString() const;				///< Get error message
	
			operator int() const;				///< Return error number

public:
	BInt32			oerrorNumber;		///< Error number
	BString			otitle;			///< The title
	BString			odescription;		///< The description
	BString			ofilename;		///< The import filename;
	BTimeStamp		ostartTime;		///< The start Time
	BTimeStamp		oendTime;		///< The end Time
	BString			onetwork;		///< The network Name
	BString			ostation;		///< The station/array name
	BString			ochannel;		///< The channel name
	BString			osource;		///< The data Source
	BString			ouser;			///< The user
};

// Bds Channel name manipulations
void	bdsChannelGetTypeAux(BString name, BString& type, BString& aux);
BString	bdsChannelGetName(BString type, BString aux);

// Bds Data Meta data manipulations
BError	bdsDataInfoSetTimeRange(DataInfo& dataInfo);
BError	bdsDataInfoFromInfo(BDictString info, DataInfo& dataInfo);
BError	bdsInfoFromDataInfo(DataInfo& dataInfo, BDictString& info);
BString bdsStationAlias(Station station);
BString	bdsDataChannelInfo(DataChannel& dataChannel);
BString	bdsDataChannelRef(DataChannel& dataChannel);
BString	bdsDataChannelRef(ChannelInfo& channelInfo);
BError	bdsDataChannelOverallResponse(ChannelInfo& channelInfo, Response& response);
BString	bdsSelectionChannelInfo(Selection& selection, BUInt channel);
BString	bdsFileNameExpand(BString fileName, ChannelInfo& channelInfo);

// BDS Metadata higher level functionality

// Calculations
double	bdsPoleZeroGain(const PoleZero& poleZero, double frequency);		///< Calculates the overal gain of the given PoleZero transfer function
void	bdsPoleZeroGainPhase(const PoleZero& poleZero, double frequency, double& gain, double& phase);	///< Calculates the overal gain and phase of the given PoleZero transfer function
void	bdsPoleZeroToFap(const PoleZero& poleZero, BUInt nPoints, double calibrationFrequency, double sampleFrequency, BArray<Fap>& fap);	///< Convert PoleZero to FAP

// Bds Debug functions
void	bdsDumpSelection(Selection sel);
void	bdsDumpDataInfo(DataInfo dataInfo, int includeInfo = 0);
void	bdsDumpChannelInfos(ChannelInfos& channelInfos);
void	bdsDumpData(DataBlock& dataBlock, int nSamples = 0);
void	bdsDumpPoleZeros(PoleZero poleZeros);

}
#endif
