/*******************************************************************************
 *	BNetworkHttp.h		BNetworkHttp class
 *			T.Barnaby,	BEAM Ltd,	2010-06-28
 *******************************************************************************
 */
#ifndef BNetworkHttp_H
#define BNetworkHttp_H	1

#include <stdint.h>
#include <BError.h>
#include <BSocket.h>
#include <BThread.h>
#include <BDict.h>
#include <BHtmlPage.h>
#include <BEvent.h>

#ifndef DBG_THREADS
#define DBG_THREADS		0x0100
#endif
#define DBG_HTTP		0x0200

typedef enum {
	HTTP_CONTINUE = 100,
	HTTP_SWITCHING_PROTOCOLS = 101,
	HTTP_OK = 200,
	HTTP_CREATED = 201,
	HTTP_ACCEPTED = 202,
	HTTP_NON_AUTHORITATIVE_INFO = 203,
	HTTP_NO_CONTENT = 204,
	HTTP_MULTIPLE_CHOICES = 300,
	HTTP_MOVED_PERMANENTLY = 301,
	HTTP_MOVED_TEMPORARILY = 302,
	HTTP_NOT_MODIFIED = 304,
	HTTP_BAD_REQUEST = 400,       /* malformed syntax */
	HTTP_UNAUTHORIZED = 401, /* authentication needed, respond with auth hdr */
	HTTP_PAYMENT_REQUIRED = 402,
	HTTP_FORBIDDEN = 403,
	HTTP_NOT_FOUND = 404,
	HTTP_REQUEST_TIMEOUT = 408,
	HTTP_INTERNAL_SERVER_ERROR = 500,
	HTTP_NOT_IMPLEMENTED = 501,   /* used for unrecognized requests */
	HTTP_BAD_GATEWAY = 502,
	HTTP_SERVICE_UNAVAILABLE = 503, /* overload, maintenance */
	HTTP_RESPONSE_SETSIZE = 0xffffffff
} HttpResponseNum;

class HttpHeaderItem {
public:
			HttpHeaderItem(BString value = "", BDictString args = BDictString()) : value(value), args(args){}
	BString		value;
	BDictString	args;
};

//typedef BDict<HttpHeaderItem> HttpHeader;

class HttpHeader : public BDict<HttpHeaderItem>{
public:
	void		dump();
};

class BNetworkHttpServer;

class BNetworkHttpConnection : public BSocket, public BThread {
public:
	enum			{ BufSize = 8192 };
	enum FormType		{ FormTypeStd, FormTypeMulti }; 

				BNetworkHttpConnection(BNetworkHttpServer& server, int fd, BSocketAddressINET from);
	virtual			~BNetworkHttpConnection();

	void			start();

protected:
	virtual BError		process();
	virtual BError		processFile();
	virtual BError		processInternal();

	BError			redirect(BString url);
	
	BError			recvLine(BString& line);
	BError			recvLine(char* data, int maxBytes, int& nRead);
	BError			recvHeader(BStringList& header);
	BError			recvHeader(HttpHeader& header);
	BError			send(const void* buf, BSize nbytes, BSize& nbytesSent, int flags = 0);
	BError			sendString(BString line);
	BError			sendHttpError(int num, BString str);
	BError			sendHeader(int num, BString contentType = "text/plain", BUInt contentLength = 0, int cache = 0, int chunked = 0, BStringList extra = BStringList());
	BString			encodeString(BString str);
	BString			decodeString(BString str, int option);
	BString			htmlStrip(BString str);

	void			cookieSet(BString name, BString value);
	BString			cookieGet(BString name);

protected:
	BNetworkHttpServer&	oserver;
	HttpHeader		oheader;
	BString			ocmd;
	BString			ourl;
	BString			opath;
	BDictString		oget;
	BDictString		oform;
	BDictString		ocookies;
	BDictString		ocookiesSend;
	
private:
	void*			function();
	BSocketAddressINET	ofrom;
	FILE*			ofile;
};

class BNetworkHttpThread : public BThread {
public:
		BNetworkHttpThread(BNetworkHttpServer& netOutput);
	void*	function();
private:
	BNetworkHttpServer&	onetOutput;
};

class BNetworkHttpServer : public BSocket {
public:
			BNetworkHttpServer();
	virtual		~BNetworkHttpServer();

	virtual BNetworkHttpConnection*	newConnection(BNetworkHttpServer& server, int fd, BSocketAddressINET from);
	
	// Main control functions		
	virtual BError		init(BString htmlDir, int port = 80);
	virtual BError		setMainPage(BHtmlPage* omainPage);
	virtual BError		setName(BString name);
	virtual BError		setTitle(BString title);
	virtual BError		setInternalFunctions(BStringList internalFunctions);
	virtual BError		start();
	virtual BError		stop();
	virtual void		clientGone(BNetworkHttpConnection* client);

	// Thread execution
	virtual BError		run();
	
	// Misc
	BHtmlPage*		mainPage();
	BString			name();
	BString			title();
	BString			htmlDir();
	BStringList&		internalFunctions();
	void			setDebug(int level);
	int			debug();

private:
	int				odebug;
	BString				oname;
	BString				ohtmlDir;
	BString				otitle;
	BNetworkHttpThread		orunThread;
	BList<BNetworkHttpConnection*>	oconnections;
	BStringList			ointernalFunctions;
	BEventInt			oclientGoneEvent;
	BHtmlPage*			omainPage;
};

#endif
